@@ -278,24 +278,26 @@ tpm2_rsa_keymgmt_gen(void *ctx, OSSL_CALLBACK *cb, void *cbarg)
             goto error1;
     }
 
-    size_t offset = 0;
-    TPM2B_TEMPLATE template = { .size = 0 };
-    r = Tss2_MU_TPMT_PUBLIC_Marshal(&gen->inPublic.publicArea,
-                                    template.buffer, sizeof(TPMT_PUBLIC), &offset);
-    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_INPUT_CORRUPTED, goto final);
-    template.size = offset;
+    TPM2B_DATA outside_info = { .size = 0 };
+    TPML_PCR_SELECTION creation_pcr = { .count = 0 };
 
-    r = Esys_CreateLoaded(gen->esys_ctx, parent,
-                          ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
-                          &gen->inSensitive, &template,
-                          &pkey->object, &keyPrivate, &keyPublic);
-    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto final);
+    /* older TPM2 chips do not support Esys_CreateLoaded */
+    r = Esys_Create(gen->esys_ctx, parent,
+                    ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
+                    &gen->inSensitive, &gen->inPublic, &outside_info, &creation_pcr,
+                    &keyPrivate, &keyPublic, NULL, NULL, NULL);
+    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error1);
 
     pkey->data.pub = *keyPublic;
-    free(keyPublic);
     pkey->data.privatetype = KEY_TYPE_BLOB;
     pkey->data.priv = *keyPrivate;
+
+    r = Esys_Load(gen->esys_ctx, parent,
+                  ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
+                  keyPrivate, keyPublic, &pkey->object);
+    free(keyPublic);
     free(keyPrivate);
+    TPM2_CHECK_RC(gen->core, r, TPM2_ERR_CANNOT_CREATE_KEY, goto error1);
 
 final:
     if (gen->parentHandle && gen->parentHandle != TPM2_RH_OWNER)
@@ -400,7 +402,7 @@ tpm2_rsa_keymgmt_get_params(void *keydata, OSSL_PARAM params[])
     if (TPM2_PKEY_RSA_SCHEME(pkey) != TPM2_ALG_NULL) {
         /* if the key is associated with a hash, it is mandatory */
         p = OSSL_PARAM_locate(params, OSSL_PKEY_PARAM_MANDATORY_DIGEST);
-        if (p != NULL && !OSSL_PARAM_set_utf8_ptr(p,
+        if (p != NULL && !OSSL_PARAM_set_utf8_string(p,
                 (char *)tpm2_hash_alg_to_name(TPM2_PKEY_RSA_HASH(pkey))))
             return 0;
     }
@@ -425,6 +427,7 @@ tpm2_rsa_keymgmt_gettable_params(void *provctx)
         OSSL_PARAM_int(OSSL_PKEY_PARAM_BITS, NULL),
         OSSL_PARAM_int(OSSL_PKEY_PARAM_SECURITY_BITS, NULL),
         OSSL_PARAM_int(OSSL_PKEY_PARAM_MAX_SIZE, NULL),
+        OSSL_PARAM_utf8_string(OSSL_PKEY_PARAM_MANDATORY_DIGEST, NULL, 0),
         /* public key */
         OSSL_PARAM_BN(OSSL_PKEY_PARAM_RSA_N, NULL, 0),
         OSSL_PARAM_BN(OSSL_PKEY_PARAM_RSA_E, NULL, 0),
